@@ -5,8 +5,8 @@ use crate::net::common::ErrorMapper;
 use crate::net::platform;
 use crate::net::socket::{Socket, SocketError};
 use crate::probe::{
-    Extensions, IcmpPacketCode, Probe, Response, ResponseData, ResponseSeq, ResponseSeqIcmp,
-    ResponseSeqTcp, ResponseSeqUdp,
+    Extensions, IcmpPacketCode, IcmpProtocolResponse, Probe, ProtocolResponse, Response,
+    ResponseData, TcpProtocolResponse, UdpProtocolResponse,
 };
 use crate::types::{PacketSize, PayloadPattern, Sequence, TraceId, TypeOfService};
 use crate::{Flags, Port, PrivilegeMode, Protocol};
@@ -82,7 +82,7 @@ impl Default for Ipv4 {
 
 impl Ipv4 {
     /// Dispatch an ICMP probe.
-    #[instrument(skip(self, icmp_send_socket, probe))]
+    #[instrument(skip(self, icmp_send_socket), level = "trace")]
     pub fn dispatch_icmp_probe<S: Socket>(
         &self,
         icmp_send_socket: &mut S,
@@ -118,7 +118,7 @@ impl Ipv4 {
     }
 
     /// Dispatch a UDP probe.
-    #[instrument(skip(self, raw_send_socket, probe))]
+    #[instrument(skip(self, raw_send_socket), level = "trace")]
     pub fn dispatch_udp_probe<S: Socket>(
         &self,
         raw_send_socket: &mut S,
@@ -142,7 +142,7 @@ impl Ipv4 {
     ///
     /// As `IP_HDRINCL` is set we must supply the IP and UDP headers which allows us to set custom
     /// values for certain fields such as the checksum as required by the Paris tracing strategy.
-    #[instrument(skip(self, raw_send_socket, probe))]
+    #[instrument(skip(self, raw_send_socket), level = "trace")]
     fn dispatch_udp_probe_raw<S: Socket>(
         &self,
         raw_send_socket: &mut S,
@@ -182,7 +182,7 @@ impl Ipv4 {
     }
 
     /// Dispatch a UDP probe using a new UDP datagram socket.
-    #[instrument(skip(self, probe))]
+    #[instrument(skip(self), level = "trace")]
     fn dispatch_udp_probe_non_raw<S: Socket>(&self, probe: Probe, payload: &[u8]) -> Result<()> {
         let local_addr = SocketAddr::new(IpAddr::V4(self.src_addr), probe.src_port.0);
         let remote_addr = SocketAddr::new(IpAddr::V4(self.dest_addr), probe.dest_port.0);
@@ -194,12 +194,13 @@ impl Ipv4 {
             .map_err(|err| ErrorMapper::addr_in_use(err, local_addr))
             .map_err(|err| ErrorMapper::probe_failed(err, ADDR_NOT_AVAILABLE_KIND))?;
         socket.set_ttl(u32::from(probe.ttl.0))?;
+        socket.set_tos(u32::from(self.tos.0))?;
         socket.send_to(payload, remote_addr)?;
         Ok(())
     }
 
     /// Dispatch a TCP probe.
-    #[instrument(skip(self, probe))]
+    #[instrument(skip(self), level = "trace")]
     pub fn dispatch_tcp_probe<S: Socket>(&self, probe: &Probe) -> Result<S> {
         let mut socket = S::new_stream_socket_ipv4()?;
         let local_addr = SocketAddr::new(IpAddr::V4(self.src_addr), probe.src_port.0);
@@ -222,7 +223,7 @@ impl Ipv4 {
     }
 
     /// Receive an ICMP probe response.
-    #[instrument(skip(self, recv_socket))]
+    #[instrument(skip(self, recv_socket), level = "trace")]
     pub fn recv_icmp_probe<S: Socket>(&self, recv_socket: &mut S) -> Result<Option<Response>> {
         let mut buf = [0_u8; MAX_PACKET_SIZE];
         match recv_socket.read(&mut buf) {
@@ -238,17 +239,18 @@ impl Ipv4 {
     }
 
     /// Receive a TCP probe response.
-    #[instrument(skip(self, tcp_socket))]
+    #[instrument(skip(self, tcp_socket), level = "trace")]
     pub fn recv_tcp_socket<S: Socket>(
         &self,
         tcp_socket: &mut S,
         src_port: Port,
         dest_port: Port,
     ) -> Result<Option<Response>> {
-        let resp_seq = ResponseSeq::Tcp(ResponseSeqTcp::new(
+        let proto_resp = ProtocolResponse::Tcp(TcpProtocolResponse::new(
             IpAddr::V4(self.dest_addr),
             src_port.0,
             dest_port.0,
+            None,
         ));
         match tcp_socket.take_error()? {
             None => {
@@ -257,7 +259,7 @@ impl Ipv4 {
                 return Ok(Some(Response::TcpReply(ResponseData::new(
                     SystemTime::now(),
                     addr,
-                    resp_seq,
+                    proto_resp,
                 ))));
             }
             Some(err) => match err {
@@ -265,24 +267,24 @@ impl Ipv4 {
                     return Ok(Some(Response::TcpRefused(ResponseData::new(
                         SystemTime::now(),
                         IpAddr::V4(self.dest_addr),
-                        resp_seq,
+                        proto_resp,
                     ))));
                 }
                 SocketError::HostUnreachable => {
                     let error_addr = tcp_socket.icmp_error_info()?;
                     return Ok(Some(Response::TimeExceeded(
-                        ResponseData::new(SystemTime::now(), error_addr, resp_seq),
+                        ResponseData::new(SystemTime::now(), error_addr, proto_resp),
                         IcmpPacketCode(1),
                         None,
                     )));
                 }
                 SocketError::Other(_) => {}
             },
-        };
+        }
         Ok(None)
     }
 
-    #[instrument(skip(self))]
+    #[instrument(skip(self), level = "trace")]
     fn extract_probe_resp(&self, ipv4: &Ipv4Packet<'_>) -> Result<Option<Response>> {
         let recv = SystemTime::now();
         let src = IpAddr::V4(ipv4.get_source());
@@ -304,13 +306,14 @@ impl Ipv4 {
                             (ipv4, None)
                         }
                     };
-                    self.extract_probe_resp_seq(&nested_ipv4)?.map(|resp_seq| {
-                        Response::TimeExceeded(
-                            ResponseData::new(recv, src, resp_seq),
-                            IcmpPacketCode(icmp_code.0),
-                            extension,
-                        )
-                    })
+                    self.extract_probe_proto_resp(&nested_ipv4)?
+                        .map(|proto_resp| {
+                            Response::TimeExceeded(
+                                ResponseData::new(recv, src, proto_resp),
+                                IcmpPacketCode(icmp_code.0),
+                                extension,
+                            )
+                        })
                 } else {
                     None
                 }
@@ -324,22 +327,24 @@ impl Ipv4 {
                     }
                     IcmpExtensionParseMode::Disabled => None,
                 };
-                self.extract_probe_resp_seq(&nested_ipv4)?.map(|resp_seq| {
-                    Response::DestinationUnreachable(
-                        ResponseData::new(recv, src, resp_seq),
-                        IcmpPacketCode(icmp_code.0),
-                        extension,
-                    )
-                })
+                self.extract_probe_proto_resp(&nested_ipv4)?
+                    .map(|proto_resp| {
+                        Response::DestinationUnreachable(
+                            ResponseData::new(recv, src, proto_resp),
+                            IcmpPacketCode(icmp_code.0),
+                            extension,
+                        )
+                    })
             }
             IcmpType::EchoReply => match self.protocol {
                 Protocol::Icmp => {
                     let packet = EchoReplyPacket::new_view(icmp_v4.packet())?;
                     let id = packet.get_identifier();
                     let seq = packet.get_sequence();
-                    let resp_seq = ResponseSeq::Icmp(ResponseSeqIcmp::new(id, seq));
+                    let proto_resp =
+                        ProtocolResponse::Icmp(IcmpProtocolResponse::new(id, seq, None));
                     Some(Response::EchoReply(
-                        ResponseData::new(recv, src, resp_seq),
+                        ResponseData::new(recv, src, proto_resp),
                         IcmpPacketCode(icmp_code.0),
                     ))
                 }
@@ -349,15 +354,17 @@ impl Ipv4 {
         })
     }
 
-    #[instrument(skip(self))]
-    fn extract_probe_resp_seq(&self, ipv4: &Ipv4Packet<'_>) -> Result<Option<ResponseSeq>> {
+    #[instrument(skip(self), level = "trace")]
+    fn extract_probe_proto_resp(&self, ipv4: &Ipv4Packet<'_>) -> Result<Option<ProtocolResponse>> {
         Ok(match (self.protocol, ipv4.get_protocol()) {
             (Protocol::Icmp, IpProtocol::Icmp) => {
                 let echo_request = extract_echo_request(ipv4)?;
                 let identifier = echo_request.get_identifier();
                 let sequence = echo_request.get_sequence();
-                Some(ResponseSeq::Icmp(ResponseSeqIcmp::new(
-                    identifier, sequence,
+                Some(ProtocolResponse::Icmp(IcmpProtocolResponse::new(
+                    identifier,
+                    sequence,
+                    Some(TypeOfService(ipv4.get_tos())),
                 )))
             }
             (Protocol::Udp, IpProtocol::Udp) => {
@@ -365,11 +372,12 @@ impl Ipv4 {
                     extract_udp_packet(ipv4)?;
                 let expected_checksum =
                     self.calc_udp_checksum(Port(src_port), Port(dest_port), payload_length)?;
-                Some(ResponseSeq::Udp(ResponseSeqUdp::new(
+                Some(ProtocolResponse::Udp(UdpProtocolResponse::new(
                     identifier,
                     IpAddr::V4(ipv4.get_destination()),
                     src_port,
                     dest_port,
+                    Some(TypeOfService(ipv4.get_tos())),
                     expected_checksum,
                     actual_checksum,
                     payload_length,
@@ -378,10 +386,11 @@ impl Ipv4 {
             }
             (Protocol::Tcp, IpProtocol::Tcp) => {
                 let (src_port, dest_port) = extract_tcp_packet(ipv4)?;
-                Some(ResponseSeq::Tcp(ResponseSeqTcp::new(
+                Some(ProtocolResponse::Tcp(TcpProtocolResponse::new(
                     IpAddr::V4(ipv4.get_destination()),
                     src_port,
                     dest_port,
+                    Some(TypeOfService(ipv4.get_tos())),
                 )))
             }
             _ => None,
@@ -450,6 +459,7 @@ impl Ipv4 {
         ipv4.set_protocol(protocol);
         ipv4.set_source(self.src_addr);
         ipv4.set_destination(self.dest_addr);
+        ipv4.set_tos(self.tos.0);
         ipv4.set_payload(payload);
         ipv4.set_identification(identification);
         ipv4.set_flags_and_fragment_offset(ipv4_flags_and_fragment_offset_header);
@@ -490,13 +500,13 @@ const fn udp_payload_size(packet_size: usize) -> usize {
     packet_size - udp_header_size - ip_header_size
 }
 
-#[instrument]
+#[instrument(level = "trace")]
 fn extract_echo_request<'a>(ipv4: &'a Ipv4Packet<'a>) -> Result<EchoRequestPacket<'a>> {
     Ok(EchoRequestPacket::new_view(ipv4.payload())?)
 }
 
 /// Get the src and dest ports from the original `UdpPacket` packet embedded in the payload.
-#[instrument]
+#[instrument(level = "trace")]
 fn extract_udp_packet(ipv4: &Ipv4Packet<'_>) -> Result<(u16, u16, u16, u16, u16)> {
     let nested = UdpPacket::new_view(ipv4.payload())?;
     Ok((
@@ -519,7 +529,7 @@ fn extract_udp_packet(ipv4: &Ipv4Packet<'_>) -> Result<(u16, u16, u16, u16, u16)
 ///
 /// We therefore have to detect this situation and ensure we provide buffer a large enough for a
 /// complete TCP packet header.
-#[instrument]
+#[instrument(level = "trace")]
 fn extract_tcp_packet(ipv4: &Ipv4Packet<'_>) -> Result<(u16, u16)> {
     let nested_tcp = ipv4.payload();
     if nested_tcp.len() < TcpPacket::minimum_packet_size() {
@@ -668,6 +678,46 @@ mod tests {
         Ok(())
     }
 
+    #[test]
+    fn test_dispatch_icmp_probe_with_tos() -> anyhow::Result<()> {
+        let probe = make_icmp_probe();
+        let src_addr = Ipv4Addr::from_str("1.2.3.4")?;
+        let dest_addr = Ipv4Addr::from_str("5.6.7.8")?;
+        let packet_size = PacketSize(28);
+        let payload_pattern = PayloadPattern(0x00);
+        let tos = TypeOfService(0xE0);
+        let byte_order = platform::Ipv4ByteOrder::Network;
+        let expected_send_to_buf = hex_literal::hex!(
+            "
+            45 e0 00 1c 00 00 40 00 0a 01 00 00 01 02 03 04
+            05 06 07 08 08 00 70 93 04 d2 82 9a
+            "
+        );
+        let expected_send_to_addr = SocketAddr::new(IpAddr::V4(dest_addr), 0);
+
+        let mut mocket = MockSocket::new();
+        mocket
+            .expect_send_to()
+            .with(
+                predicate::eq(expected_send_to_buf),
+                predicate::eq(expected_send_to_addr),
+            )
+            .times(1)
+            .returning(|_, _| Ok(()));
+
+        let ipv4 = Ipv4 {
+            src_addr,
+            dest_addr,
+            byte_order,
+            packet_size,
+            payload_pattern,
+            tos,
+            ..Default::default()
+        };
+        ipv4.dispatch_icmp_probe(&mut mocket, probe)?;
+        Ok(())
+    }
+
     #[test]
     fn test_dispatch_udp_probe_classic_privileged_no_payload() -> anyhow::Result<()> {
         let probe = make_udp_probe(123, 456);
@@ -853,6 +903,7 @@ mod tests {
         let expected_send_to_addr = SocketAddr::new(IpAddr::V4(dest_addr), 456);
         let expected_bind_addr = SocketAddr::new(IpAddr::V4(src_addr), 123);
         let expected_set_ttl = 10;
+        let expected_set_tos = 0;
 
         let mut mocket = MockSocket::new();
 
@@ -871,6 +922,12 @@ mod tests {
                 .times(1)
                 .returning(|_| Ok(()));
 
+            mocket
+                .expect_set_tos()
+                .with(predicate::eq(expected_set_tos))
+                .times(1)
+                .returning(|_| Ok(()));
+
             mocket
                 .expect_send_to()
                 .with(
@@ -909,6 +966,7 @@ mod tests {
         let expected_send_to_addr = SocketAddr::new(IpAddr::V4(dest_addr), 456);
         let expected_bind_addr = SocketAddr::new(IpAddr::V4(src_addr), 123);
         let expected_set_ttl = 10;
+        let expected_set_tos = 0;
 
         let mut mocket = MockSocket::new();
 
@@ -927,6 +985,12 @@ mod tests {
                 .times(1)
                 .returning(|_| Ok(()));
 
+            mocket
+                .expect_set_tos()
+                .with(predicate::eq(expected_set_tos))
+                .times(1)
+                .returning(|_| Ok(()));
+
             mocket
                 .expect_send_to()
                 .with(
@@ -951,6 +1015,113 @@ mod tests {
         Ok(())
     }
 
+    #[test]
+    fn test_dispatch_udp_probe_classic_privileged_with_tos() -> anyhow::Result<()> {
+        let probe = make_udp_probe(123, 456);
+        let src_addr = Ipv4Addr::from_str("1.2.3.4")?;
+        let dest_addr = Ipv4Addr::from_str("5.6.7.8")?;
+        let privilege_mode = PrivilegeMode::Privileged;
+        let packet_size = PacketSize(28);
+        let payload_pattern = PayloadPattern(0x00);
+        let byte_order = platform::Ipv4ByteOrder::Network;
+        let tos = TypeOfService(0xE0);
+        let expected_send_to_buf = hex_literal::hex!(
+            "
+            45 e0 00 1c 04 d2 40 00 0a 11 00 00 01 02 03 04
+            05 06 07 08 00 7b 01 c8 00 08 ed 87
+            "
+        );
+        let expected_send_to_addr = SocketAddr::new(IpAddr::V4(dest_addr), 456);
+
+        let mut mocket = MockSocket::new();
+        mocket
+            .expect_send_to()
+            .with(
+                predicate::eq(expected_send_to_buf),
+                predicate::eq(expected_send_to_addr),
+            )
+            .times(1)
+            .returning(|_, _| Ok(()));
+
+        let ipv4 = Ipv4 {
+            src_addr,
+            dest_addr,
+            byte_order,
+            packet_size,
+            payload_pattern,
+            privilege_mode,
+            tos,
+            ..Default::default()
+        };
+        ipv4.dispatch_udp_probe(&mut mocket, probe)?;
+        Ok(())
+    }
+
+    #[test]
+    fn test_dispatch_udp_probe_classic_unprivileged_with_tos() -> anyhow::Result<()> {
+        let _m = MTX.lock();
+        let probe = make_udp_probe(123, 456);
+        let src_addr = Ipv4Addr::from_str("1.2.3.4")?;
+        let dest_addr = Ipv4Addr::from_str("5.6.7.8")?;
+        let privilege_mode = PrivilegeMode::Unprivileged;
+        let packet_size = PacketSize(28);
+        let payload_pattern = PayloadPattern(0x00);
+        let byte_order = platform::Ipv4ByteOrder::Network;
+        let tos = TypeOfService(224);
+        let expected_send_to_buf = hex_literal::hex!("");
+        let expected_send_to_addr = SocketAddr::new(IpAddr::V4(dest_addr), 456);
+        let expected_bind_addr = SocketAddr::new(IpAddr::V4(src_addr), 123);
+        let expected_set_ttl = 10;
+        let expected_set_tos = u32::from(tos.0);
+
+        let mut mocket = MockSocket::new();
+
+        let ctx = MockSocket::new_udp_send_socket_ipv4_context();
+        ctx.expect().with(predicate::eq(false)).returning(move |_| {
+            let mut mocket = MockSocket::new();
+            mocket
+                .expect_bind()
+                .with(predicate::eq(expected_bind_addr))
+                .times(1)
+                .returning(|_| Ok(()));
+
+            mocket
+                .expect_set_ttl()
+                .with(predicate::eq(expected_set_ttl))
+                .times(1)
+                .returning(|_| Ok(()));
+
+            mocket
+                .expect_set_tos()
+                .with(predicate::eq(expected_set_tos))
+                .times(1)
+                .returning(|_| Ok(()));
+
+            mocket
+                .expect_send_to()
+                .with(
+                    predicate::eq(expected_send_to_buf),
+                    predicate::eq(expected_send_to_addr),
+                )
+                .times(1)
+                .returning(|_, _| Ok(()));
+
+            Ok(mocket)
+        });
+        let ipv4 = Ipv4 {
+            src_addr,
+            dest_addr,
+            byte_order,
+            packet_size,
+            payload_pattern,
+            privilege_mode,
+            tos,
+            ..Default::default()
+        };
+        ipv4.dispatch_udp_probe(&mut mocket, probe)?;
+        Ok(())
+    }
+
     #[test]
     fn test_dispatch_udp_probe_invalid_packet_size_low() -> anyhow::Result<()> {
         let probe = make_udp_probe(123, 456);
@@ -1005,10 +1176,10 @@ mod tests {
         let probe = make_udp_probe(123, 456);
         let src_addr = Ipv4Addr::from_str("1.2.3.4")?;
         let dest_addr = Ipv4Addr::from_str("5.6.7.8")?;
-        let tos = TypeOfService(0);
+        let tos = TypeOfService(224);
         let expected_bind_addr = SocketAddr::new(IpAddr::V4(src_addr), 123);
         let expected_set_ttl = 10;
-        let expected_set_tos = 0;
+        let expected_set_tos = u32::from(tos.0);
         let expected_connect_addr = SocketAddr::new(IpAddr::V4(dest_addr), 456);
 
         let ctx = MockSocket::new_stream_socket_ipv4_context();
@@ -1078,10 +1249,11 @@ mod tests {
         let Response::EchoReply(
             ResponseData {
                 addr,
-                resp_seq:
-                    ResponseSeq::Icmp(ResponseSeqIcmp {
+                proto_resp:
+                    ProtocolResponse::Icmp(IcmpProtocolResponse {
                         identifier,
                         sequence,
+                        tos,
                     }),
                 ..
             },
@@ -1090,12 +1262,10 @@ mod tests {
         else {
             panic!("expected EchoReply")
         };
-        assert_eq!(
-            IpAddr::V4(Ipv4Addr::from_str("142.251.222.206").unwrap()),
-            addr
-        );
+        assert_eq!(IpAddr::V4(Ipv4Addr::from_str("142.251.222.206")?), addr);
         assert_eq!(30167, identifier);
         assert_eq!(33049, sequence);
+        assert_eq!(None, tos);
         assert_eq!(IcmpPacketCode(0), icmp_code);
         Ok(())
     }
@@ -1128,10 +1298,11 @@ mod tests {
         let Response::TimeExceeded(
             ResponseData {
                 addr,
-                resp_seq:
-                    ResponseSeq::Icmp(ResponseSeqIcmp {
+                proto_resp:
+                    ProtocolResponse::Icmp(IcmpProtocolResponse {
                         identifier,
                         sequence,
+                        tos,
                     }),
                 ..
             },
@@ -1141,12 +1312,10 @@ mod tests {
         else {
             panic!("expected TimeExceeded")
         };
-        assert_eq!(
-            IpAddr::V4(Ipv4Addr::from_str("142.250.61.129").unwrap()),
-            addr
-        );
+        assert_eq!(IpAddr::V4(Ipv4Addr::from_str("142.250.61.129")?), addr);
         assert_eq!(30167, identifier);
         assert_eq!(33047, sequence);
+        assert_eq!(Some(TypeOfService(96)), tos);
         assert_eq!(IcmpPacketCode(0), icmp_code);
         assert_eq!(None, extensions);
         Ok(())
@@ -1177,10 +1346,11 @@ mod tests {
         let Response::DestinationUnreachable(
             ResponseData {
                 addr,
-                resp_seq:
-                    ResponseSeq::Icmp(ResponseSeqIcmp {
+                proto_resp:
+                    ProtocolResponse::Icmp(IcmpProtocolResponse {
                         identifier,
                         sequence,
+                        tos,
                     }),
                 ..
             },
@@ -1190,9 +1360,10 @@ mod tests {
         else {
             panic!("expected DestinationUnreachable")
         };
-        assert_eq!(IpAddr::V4(Ipv4Addr::from_str("20.0.0.254").unwrap()), addr);
+        assert_eq!(IpAddr::V4(Ipv4Addr::from_str("20.0.0.254")?), addr);
         assert_eq!(31489, identifier);
         assert_eq!(33060, sequence);
+        assert_eq!(Some(TypeOfService(0)), tos);
         assert_eq!(IcmpPacketCode(1), icmp_code);
         assert_eq!(None, extensions);
         Ok(())
@@ -1218,8 +1389,8 @@ mod tests {
             .returning(mocket_read!(expected_read_buf));
         let ipv4 = Ipv4 {
             protocol: Protocol::Udp,
-            src_addr: Ipv4Addr::from_str("192.168.1.21").unwrap(),
-            dest_addr: Ipv4Addr::from_str("142.250.204.142").unwrap(),
+            src_addr: Ipv4Addr::from_str("192.168.1.21")?,
+            dest_addr: Ipv4Addr::from_str("142.250.204.142")?,
             icmp_extension_mode: IcmpExtensionParseMode::Disabled,
             ..Default::default()
         };
@@ -1228,12 +1399,13 @@ mod tests {
         let Response::TimeExceeded(
             ResponseData {
                 addr,
-                resp_seq:
-                    ResponseSeq::Udp(ResponseSeqUdp {
+                proto_resp:
+                    ProtocolResponse::Udp(UdpProtocolResponse {
                         identifier,
                         dest_addr,
                         src_port,
                         dest_port,
+                        tos,
                         expected_udp_checksum,
                         actual_udp_checksum,
                         payload_len,
@@ -1247,14 +1419,15 @@ mod tests {
         else {
             panic!("expected TimeExceeded")
         };
-        assert_eq!(IpAddr::V4(Ipv4Addr::from_str("192.168.1.1").unwrap()), addr);
+        assert_eq!(IpAddr::V4(Ipv4Addr::from_str("192.168.1.1")?), addr);
         assert_eq!(36969, identifier);
         assert_eq!(
-            IpAddr::V4(Ipv4Addr::from_str("142.250.204.142").unwrap()),
+            IpAddr::V4(Ipv4Addr::from_str("142.250.204.142")?),
             dest_addr
         );
         assert_eq!(31829, src_port);
         assert_eq!(33030, dest_port);
+        assert_eq!(Some(TypeOfService(0)), tos);
         assert_eq!(58571, expected_udp_checksum);
         assert_eq!(58571, actual_udp_checksum);
         assert_eq!(56, payload_len);
@@ -1284,8 +1457,8 @@ mod tests {
             .returning(mocket_read!(expected_read_buf));
         let ipv4 = Ipv4 {
             protocol: Protocol::Udp,
-            src_addr: Ipv4Addr::from_str("192.168.1.21").unwrap(),
-            dest_addr: Ipv4Addr::from_str("9.9.9.9").unwrap(),
+            src_addr: Ipv4Addr::from_str("192.168.1.21")?,
+            dest_addr: Ipv4Addr::from_str("9.9.9.9")?,
             icmp_extension_mode: IcmpExtensionParseMode::Disabled,
             ..Default::default()
         };
@@ -1294,12 +1467,13 @@ mod tests {
         let Response::DestinationUnreachable(
             ResponseData {
                 addr,
-                resp_seq:
-                    ResponseSeq::Udp(ResponseSeqUdp {
+                proto_resp:
+                    ProtocolResponse::Udp(UdpProtocolResponse {
                         identifier,
                         dest_addr,
                         src_port,
                         dest_port,
+                        tos,
                         expected_udp_checksum,
                         actual_udp_checksum,
                         payload_len,
@@ -1313,14 +1487,12 @@ mod tests {
         else {
             panic!("expected DestinationUnreachable")
         };
-        assert_eq!(IpAddr::V4(Ipv4Addr::from_str("9.9.9.9").unwrap()), addr);
+        assert_eq!(IpAddr::V4(Ipv4Addr::from_str("9.9.9.9")?), addr);
         assert_eq!(41481, identifier);
-        assert_eq!(
-            IpAddr::V4(Ipv4Addr::from_str("9.9.9.9").unwrap()),
-            dest_addr
-        );
+        assert_eq!(IpAddr::V4(Ipv4Addr::from_str("9.9.9.9")?), dest_addr);
         assert_eq!(32779, src_port);
         assert_eq!(33010, dest_port);
+        assert_eq!(Some(TypeOfService(32)), tos);
         assert_eq!(10913, expected_udp_checksum);
         assert_eq!(10913, actual_udp_checksum);
         assert_eq!(56, payload_len);
@@ -1357,11 +1529,12 @@ mod tests {
         let Response::TimeExceeded(
             ResponseData {
                 addr,
-                resp_seq:
-                    ResponseSeq::Tcp(ResponseSeqTcp {
+                proto_resp:
+                    ProtocolResponse::Tcp(TcpProtocolResponse {
                         dest_addr,
                         src_port,
                         dest_port,
+                        tos,
                     }),
                 ..
             },
@@ -1371,16 +1544,14 @@ mod tests {
         else {
             panic!("expected TimeExceeded")
         };
+        assert_eq!(IpAddr::V4(Ipv4Addr::from_str("209.85.240.235")?), addr);
         assert_eq!(
-            IpAddr::V4(Ipv4Addr::from_str("209.85.240.235").unwrap()),
-            addr
-        );
-        assert_eq!(
-            IpAddr::V4(Ipv4Addr::from_str("142.250.204.142").unwrap()),
+            IpAddr::V4(Ipv4Addr::from_str("142.250.204.142")?),
             dest_addr
         );
         assert_eq!(33021, src_port);
         assert_eq!(80, dest_port);
+        assert_eq!(Some(TypeOfService(128)), tos);
         assert_eq!(IcmpPacketCode(0), icmp_code);
         assert_eq!(None, extensions);
         Ok(())
@@ -1413,11 +1584,12 @@ mod tests {
         let Response::DestinationUnreachable(
             ResponseData {
                 addr,
-                resp_seq:
-                    ResponseSeq::Tcp(ResponseSeqTcp {
+                proto_resp:
+                    ProtocolResponse::Tcp(TcpProtocolResponse {
                         dest_addr,
                         src_port,
                         dest_port,
+                        tos,
                     }),
                 ..
             },
@@ -1427,13 +1599,11 @@ mod tests {
         else {
             panic!("expected DestinationUnreachable")
         };
-        assert_eq!(IpAddr::V4(Ipv4Addr::from_str("9.9.9.9").unwrap()), addr);
-        assert_eq!(
-            IpAddr::V4(Ipv4Addr::from_str("9.9.9.9").unwrap()),
-            dest_addr
-        );
+        assert_eq!(IpAddr::V4(Ipv4Addr::from_str("9.9.9.9")?), addr);
+        assert_eq!(IpAddr::V4(Ipv4Addr::from_str("9.9.9.9")?), dest_addr);
         assert_eq!(33010, src_port);
         assert_eq!(10011, dest_port);
+        assert_eq!(Some(TypeOfService(32)), tos);
         assert_eq!(IcmpPacketCode(10), icmp_code);
         assert_eq!(None, extensions);
         Ok(())
@@ -1596,11 +1766,12 @@ mod tests {
 
         let Response::TcpReply(ResponseData {
             addr,
-            resp_seq:
-                ResponseSeq::Tcp(ResponseSeqTcp {
+            proto_resp:
+                ProtocolResponse::Tcp(TcpProtocolResponse {
                     dest_addr,
                     src_port,
                     dest_port,
+                    tos,
                 }),
             ..
         }) = resp
@@ -1610,6 +1781,7 @@ mod tests {
         assert_eq!(dest_addr, addr);
         assert_eq!(33434, src_port);
         assert_eq!(456, dest_port);
+        assert_eq!(None, tos);
         Ok(())
     }
 
@@ -1633,11 +1805,12 @@ mod tests {
 
         let Response::TcpRefused(ResponseData {
             addr,
-            resp_seq:
-                ResponseSeq::Tcp(ResponseSeqTcp {
+            proto_resp:
+                ProtocolResponse::Tcp(TcpProtocolResponse {
                     dest_addr,
                     src_port,
                     dest_port,
+                    tos,
                 }),
             ..
         }) = resp
@@ -1647,6 +1820,7 @@ mod tests {
         assert_eq!(dest_addr, addr);
         assert_eq!(33434, src_port);
         assert_eq!(80, dest_port);
+        assert_eq!(None, tos);
         Ok(())
     }
 
@@ -1675,11 +1849,12 @@ mod tests {
         let Response::TimeExceeded(
             ResponseData {
                 addr,
-                resp_seq:
-                    ResponseSeq::Tcp(ResponseSeqTcp {
+                proto_resp:
+                    ProtocolResponse::Tcp(TcpProtocolResponse {
                         dest_addr,
                         src_port,
                         dest_port,
+                        tos,
                     }),
                 ..
             },
@@ -1692,6 +1867,7 @@ mod tests {
         assert_eq!(dest_addr, addr);
         assert_eq!(33434, src_port);
         assert_eq!(80, dest_port);
+        assert_eq!(None, tos);
         assert_eq!(IcmpPacketCode(1), icmp_code);
         assert_eq!(None, extensions);
         Ok(())
@@ -1758,8 +1934,8 @@ mod tests {
             .returning(mocket_read!(expected_read_buf));
         let ipv4 = Ipv4 {
             protocol: Protocol::Udp,
-            src_addr: Ipv4Addr::from_str("192.168.1.21").unwrap(),
-            dest_addr: Ipv4Addr::from_str("9.9.9.9").unwrap(),
+            src_addr: Ipv4Addr::from_str("192.168.1.21")?,
+            dest_addr: Ipv4Addr::from_str("9.9.9.9")?,
             icmp_extension_mode: IcmpExtensionParseMode::Disabled,
             ..Default::default()
         };
@@ -1768,12 +1944,13 @@ mod tests {
         let Response::TimeExceeded(
             ResponseData {
                 addr,
-                resp_seq:
-                    ResponseSeq::Udp(ResponseSeqUdp {
+                proto_resp:
+                    ProtocolResponse::Udp(UdpProtocolResponse {
                         identifier,
                         dest_addr,
                         src_port,
                         dest_port,
+                        tos,
                         expected_udp_checksum,
                         actual_udp_checksum,
                         payload_len,
@@ -1787,14 +1964,15 @@ mod tests {
         else {
             panic!("expected TimeExceeded")
         };
-        assert_eq!(IpAddr::V4(Ipv4Addr::from_str("192.168.1.1").unwrap()), addr);
+        assert_eq!(IpAddr::V4(Ipv4Addr::from_str("192.168.1.1")?), addr);
         assert_eq!(36969, identifier);
         assert_eq!(
-            IpAddr::V4(Ipv4Addr::from_str("142.250.204.142").unwrap()),
+            IpAddr::V4(Ipv4Addr::from_str("142.250.204.142")?),
             dest_addr
         );
         assert_eq!(31829, src_port);
         assert_eq!(33030, dest_port);
+        assert_eq!(Some(TypeOfService(0)), tos);
         assert_eq!(9963, expected_udp_checksum);
         assert_eq!(58571, actual_udp_checksum);
         assert_eq!(2040, payload_len);
