@@ -82,7 +82,7 @@ impl Platform for PlatformImpl {
     }
 }
 
-#[instrument]
+#[instrument(level = "trace")]
 pub fn startup() -> Result<()> {
     SocketImpl::startup().map_err(Error::IoError)
 }
@@ -94,8 +94,9 @@ const WINSOCK_VERSION: u16 = 0x202;
 pub struct SocketImpl {
     inner: socket2::Socket,
     ol: Box<OVERLAPPED>,
-    buf: Vec<u8>,
+    buf: Box<[u8]>,
     from: Box<SOCKADDR_STORAGE>,
+    from_len: i32,
     bytes_read: u32,
 }
 
@@ -114,25 +115,27 @@ impl SocketImpl {
         let inner = socket2::Socket::new(domain, ty, protocol)
             .map_err(|err| IoError::Other(err, IoOperation::NewSocket))?;
         let from = Box::new(Self::new_sockaddr_storage());
+        let from_len = std::mem::size_of::<SOCKADDR_STORAGE>() as i32;
         let ol = Box::new(Self::new_overlapped());
-        let buf = vec![0u8; MAX_PACKET_SIZE];
+        let buf = Box::new([0; MAX_PACKET_SIZE]);
         Ok(Self {
             inner,
             ol,
             buf,
             from,
+            from_len,
             bytes_read: 0,
         })
     }
 
-    #[instrument(skip(self))]
+    #[instrument(skip(self), level = "trace")]
     fn create_event(&mut self) -> IoResult<()> {
         self.ol.hEvent = syscall!(WSACreateEvent(), |res| { res == 0 || res == -1 })
             .map_err(|err| IoError::Other(err, IoOperation::WSACreateEvent))?;
         Ok(())
     }
 
-    #[instrument(skip(self))]
+    #[instrument(skip(self), level = "trace")]
     fn wait_for_event(&self, timeout: Duration) -> IoResult<bool> {
         let millis = timeout.as_millis() as u32;
         let rc = syscall_threading!(WaitForSingleObject(self.ol.hEvent, millis));
@@ -147,14 +150,14 @@ impl SocketImpl {
         Ok(true)
     }
 
-    #[instrument(skip(self))]
+    #[instrument(skip(self), level = "trace")]
     fn reset_event(&self) -> IoResult<()> {
         syscall!(WSAResetEvent(self.ol.hEvent), |res| { res == 0 })
             .map_err(|err| IoError::Other(err, IoOperation::WSAResetEvent))
             .map(|_| ())
     }
 
-    #[instrument(skip(self, optval))]
+    #[instrument(skip(self, optval), level = "trace")]
     fn getsockopt<T>(&self, level: i32, optname: i32, mut optval: T) -> StdIoResult<T> {
         let mut optlen = size_of::<T>() as i32;
         syscall!(
@@ -170,7 +173,7 @@ impl SocketImpl {
         Ok(optval)
     }
 
-    #[instrument(skip(self))]
+    #[instrument(skip(self), level = "trace")]
     fn setsockopt_u32(&self, level: i32, optname: i32, optval: u32) -> StdIoResult<()> {
         let bytes = optval.to_ne_bytes();
         let optval = addr_of!(bytes).cast();
@@ -187,18 +190,18 @@ impl SocketImpl {
         .map(|_| ())
     }
 
-    #[instrument(skip(self))]
+    #[instrument(skip(self), level = "trace")]
     fn setsockopt_bool(&self, level: i32, optname: i32, optval: bool) -> StdIoResult<()> {
         self.setsockopt_u32(level, optname, u32::from(optval))
     }
 
-    #[instrument(skip(self))]
+    #[instrument(skip(self), level = "trace")]
     fn set_fail_connect_on_icmp_error(&self, enabled: bool) -> IoResult<()> {
         self.setsockopt_bool(IPPROTO_TCP, TCP_FAIL_CONNECT_ON_ICMP_ERROR as _, enabled)
             .map_err(|err| IoError::Other(err, IoOperation::SetTcpFailConnectOnIcmpError))
     }
 
-    #[instrument(skip(self))]
+    #[instrument(skip(self), level = "trace")]
     fn set_non_blocking(&self, is_non_blocking: bool) -> IoResult<()> {
         self.inner
             .set_nonblocking(is_non_blocking)
@@ -206,13 +209,12 @@ impl SocketImpl {
     }
 
     // TODO handle case where `WSARecvFrom` succeeded immediately.
-    #[instrument(skip(self))]
+    #[instrument(skip(self), level = "trace")]
     fn post_recv_from(&mut self) -> IoResult<()> {
         fn is_err(res: i32) -> bool {
             res == SOCKET_ERROR
                 && StdIoError::last_os_error().raw_os_error() != Some(WSA_IO_PENDING)
         }
-        let mut fromlen = std::mem::size_of::<SOCKADDR_STORAGE>() as i32;
         let wbuf = WSABUF {
             len: MAX_PACKET_SIZE as u32,
             buf: self.buf.as_mut_ptr(),
@@ -225,7 +227,7 @@ impl SocketImpl {
                 null_mut(),
                 &mut 0,
                 addr_of_mut!(*self.from).cast(),
-                addr_of_mut!(fromlen),
+                addr_of_mut!(self.from_len),
                 addr_of_mut!(*self.ol),
                 None,
             ),
@@ -235,7 +237,7 @@ impl SocketImpl {
         Ok(())
     }
 
-    #[instrument(skip(self))]
+    #[instrument(skip(self), level = "trace")]
     fn get_overlapped_result(&mut self) -> IoResult<()> {
         let mut bytes_read = 0;
         let mut flags = 0;
@@ -283,7 +285,6 @@ impl SocketImpl {
 #[allow(clippy::redundant_closure_call)]
 impl Drop for SocketImpl {
     fn drop(&mut self) {
-        self.close().unwrap_or_default();
         if self.ol.hEvent != -1 && self.ol.hEvent != 0 {
             syscall!(WSACloseEvent(self.ol.hEvent), |res| { res == 0 }).unwrap_or_default();
         }
@@ -292,7 +293,7 @@ impl Drop for SocketImpl {
 
 #[allow(clippy::cast_possible_wrap, clippy::redundant_closure_call)]
 impl Socket for SocketImpl {
-    #[instrument]
+    #[instrument(level = "trace")]
     fn new_icmp_send_socket_ipv4(raw: bool) -> IoResult<Self> {
         if raw {
             let mut sock = Self::new(Domain::IPV4, Type::RAW, Some(Protocol::from(IPPROTO_RAW)))?;
@@ -304,7 +305,7 @@ impl Socket for SocketImpl {
         }
     }
 
-    #[instrument]
+    #[instrument(level = "trace")]
     fn new_icmp_send_socket_ipv6(raw: bool) -> IoResult<Self> {
         if raw {
             let sock = Self::new(Domain::IPV6, Type::RAW, Some(Protocol::ICMPV6))?;
@@ -315,7 +316,7 @@ impl Socket for SocketImpl {
         }
     }
 
-    #[instrument]
+    #[instrument(level = "trace")]
     fn new_udp_send_socket_ipv4(raw: bool) -> IoResult<Self> {
         if raw {
             let mut sock = Self::new(Domain::IPV4, Type::RAW, Some(Protocol::from(IPPROTO_RAW)))?;
@@ -327,7 +328,7 @@ impl Socket for SocketImpl {
         }
     }
 
-    #[instrument]
+    #[instrument(level = "trace")]
     fn new_udp_send_socket_ipv6(raw: bool) -> IoResult<Self> {
         if raw {
             let sock = Self::new(Domain::IPV6, Type::RAW, Some(Protocol::UDP))?;
@@ -338,7 +339,7 @@ impl Socket for SocketImpl {
         }
     }
 
-    #[instrument]
+    #[instrument(level = "trace")]
     fn new_recv_socket_ipv4(src_addr: Ipv4Addr, raw: bool) -> IoResult<Self> {
         if raw {
             let mut sock = Self::new(Domain::IPV4, Type::RAW, Some(Protocol::ICMPV4))?;
@@ -352,7 +353,7 @@ impl Socket for SocketImpl {
         }
     }
 
-    #[instrument]
+    #[instrument(level = "trace")]
     fn new_recv_socket_ipv6(src_addr: Ipv6Addr, raw: bool) -> IoResult<Self> {
         if raw {
             let mut sock = Self::new(Domain::IPV6, Type::RAW, Some(Protocol::ICMPV6))?;
@@ -365,7 +366,7 @@ impl Socket for SocketImpl {
         }
     }
 
-    #[instrument]
+    #[instrument(level = "trace")]
     fn new_stream_socket_ipv4() -> IoResult<Self> {
         let mut sock = Self::new(Domain::IPV4, Type::STREAM, Some(Protocol::TCP))?;
         sock.set_non_blocking(true)?;
@@ -373,7 +374,7 @@ impl Socket for SocketImpl {
         Ok(sock)
     }
 
-    #[instrument]
+    #[instrument(level = "trace")]
     fn new_stream_socket_ipv6() -> IoResult<Self> {
         let mut sock = Self::new(Domain::IPV6, Type::STREAM, Some(Protocol::TCP))?;
         sock.set_non_blocking(true)?;
@@ -381,17 +382,17 @@ impl Socket for SocketImpl {
         Ok(sock)
     }
 
-    #[instrument]
+    #[instrument(level = "trace")]
     fn new_udp_dgram_socket_ipv4() -> IoResult<Self> {
         Self::new(Domain::IPV4, Type::DGRAM, Some(Protocol::UDP))
     }
 
-    #[instrument]
+    #[instrument(level = "trace")]
     fn new_udp_dgram_socket_ipv6() -> IoResult<Self> {
         Self::new(Domain::IPV6, Type::DGRAM, Some(Protocol::UDP))
     }
 
-    #[instrument(skip(self))]
+    #[instrument(skip(self), level = "trace")]
     fn bind(&mut self, addr: SocketAddr) -> IoResult<()> {
         self.inner
             .bind(&SockAddr::from(addr))
@@ -407,21 +408,28 @@ impl Socket for SocketImpl {
         Ok(())
     }
 
-    #[instrument(skip(self))]
+    #[instrument(skip(self), level = "trace")]
     fn set_tos(&mut self, tos: u32) -> IoResult<()> {
         self.inner
             .set_tos(tos)
             .map_err(|err| IoError::Other(err, IoOperation::SetTos))
     }
 
-    #[instrument(skip(self))]
+    fn set_tclass_v6(&mut self, tclass: u32) -> IoResult<()> {
+        if tclass > 0 {
+            unimplemented!("setting tclass_v6 is not supported on Windows")
+        }
+        Ok(())
+    }
+
+    #[instrument(skip(self), level = "trace")]
     fn set_ttl(&mut self, ttl: u32) -> IoResult<()> {
         self.inner
             .set_ttl(ttl)
             .map_err(|err| IoError::Other(err, IoOperation::SetTtl))
     }
 
-    #[instrument(skip(self))]
+    #[instrument(skip(self), level = "trace")]
     fn set_reuse_port(&mut self, is_reuse_port: bool) -> IoResult<()> {
         self.setsockopt_bool(SOL_SOCKET as _, SO_REUSE_UNICASTPORT as _, is_reuse_port)
             .or_else(|_| {
@@ -430,21 +438,21 @@ impl Socket for SocketImpl {
             .map_err(|err| IoError::Other(err, IoOperation::SetReusePort))
     }
 
-    #[instrument(skip(self))]
+    #[instrument(skip(self), level = "trace")]
     fn set_header_included(&mut self, is_header_included: bool) -> IoResult<()> {
         self.inner
             .set_header_included_v4(is_header_included)
             .map_err(|err| IoError::Other(err, IoOperation::SetHeaderIncluded))
     }
 
-    #[instrument(skip(self))]
+    #[instrument(skip(self), level = "trace")]
     fn set_unicast_hops_v6(&mut self, max_hops: u8) -> IoResult<()> {
         self.inner
             .set_unicast_hops_v6(max_hops.into())
             .map_err(|err| IoError::Other(err, IoOperation::SetUnicastHopsV6))
     }
 
-    #[instrument(skip(self))]
+    #[instrument(skip(self), level = "trace")]
     fn connect(&mut self, addr: SocketAddr) -> IoResult<()> {
         self.set_fail_connect_on_icmp_error(true)?;
         syscall!(
@@ -464,20 +472,20 @@ impl Socket for SocketImpl {
         }
     }
 
-    #[instrument(skip(self, buf))]
+    #[instrument(skip(self, buf), level = "trace")]
     fn send_to(&mut self, buf: &[u8], addr: SocketAddr) -> IoResult<()> {
-        tracing::debug!(buf = format!("{:02x?}", buf.iter().format(" ")), ?addr);
+        tracing::trace!(buf = format!("{:02x?}", buf.iter().format(" ")), ?addr);
         self.inner
             .send_to(buf, &SockAddr::from(addr))
             .map_err(|err| IoError::SendTo(err, addr))?;
         Ok(())
     }
 
-    #[instrument(skip(self))]
+    #[instrument(skip(self), level = "trace")]
     fn is_readable(&mut self, timeout: Duration) -> IoResult<bool> {
         if !self.wait_for_event(timeout)? {
             return Ok(false);
-        };
+        }
         while let Err(err) = self.get_overlapped_result() {
             if err.kind() != ErrorKind::Std(StdIoError::from_raw_os_error(WSA_IO_INCOMPLETE).kind())
             {
@@ -488,11 +496,11 @@ impl Socket for SocketImpl {
         Ok(true)
     }
 
-    #[instrument(skip(self))]
+    #[instrument(skip(self), level = "trace")]
     fn is_writable(&mut self) -> IoResult<bool> {
         if !self.wait_for_event(Duration::ZERO)? {
             return Ok(false);
-        };
+        }
         while let Err(err) = self.get_overlapped_result() {
             if err.kind() != ErrorKind::Std(StdIoError::from_raw_os_error(WSA_IO_INCOMPLETE).kind())
             {
@@ -503,12 +511,12 @@ impl Socket for SocketImpl {
         Ok(true)
     }
 
-    #[instrument(skip(self, buf), ret)]
+    #[instrument(skip(self, buf), level = "trace")]
     fn recv_from(&mut self, buf: &mut [u8]) -> IoResult<(usize, Option<SocketAddr>)> {
         let addr = sockaddrptr_to_ipaddr(addr_of_mut!(*self.from))
             .map_err(|err| IoError::Other(err, IoOperation::RecvFrom))?;
         let len = self.read(buf)?;
-        tracing::debug!(
+        tracing::trace!(
             buf = format!("{:02x?}", buf[..len].iter().format(" ")),
             len,
             ?addr
@@ -516,23 +524,23 @@ impl Socket for SocketImpl {
         Ok((len, Some(SocketAddr::new(addr, 0))))
     }
 
-    #[instrument(skip(self, buf), ret)]
+    #[instrument(skip(self, buf), ret, level = "trace")]
     fn read(&mut self, buf: &mut [u8]) -> IoResult<usize> {
-        buf.copy_from_slice(self.buf.as_slice());
-        let bytes_read = self.bytes_read as usize;
-        tracing::debug!(buf = format!("{:02x?}", buf[..bytes_read].iter().format(" ")));
+        let bytes_read = std::cmp::min(self.bytes_read as usize, buf.len());
+        buf[..bytes_read].copy_from_slice(&self.buf[..bytes_read]);
+        tracing::trace!(buf = format!("{:02x?}", buf[..bytes_read].iter().format(" ")));
         self.post_recv_from()?;
         Ok(bytes_read)
     }
 
-    #[instrument(skip(self))]
+    #[instrument(skip(self), level = "trace")]
     fn shutdown(&mut self) -> IoResult<()> {
         self.inner
             .shutdown(std::net::Shutdown::Both)
             .map_err(|err| IoError::Other(err, IoOperation::Shutdown))
     }
 
-    #[instrument(skip(self), ret)]
+    #[instrument(skip(self), ret, level = "trace")]
     fn peer_addr(&mut self) -> IoResult<Option<SocketAddr>> {
         Ok(self
             .inner
@@ -541,7 +549,7 @@ impl Socket for SocketImpl {
             .as_socket())
     }
 
-    #[instrument(skip(self), ret)]
+    #[instrument(skip(self), ret, level = "trace")]
     fn take_error(&mut self) -> IoResult<Option<SocketError>> {
         match self.getsockopt(SOL_SOCKET as _, SO_ERROR as _, 0) {
             Ok(0) => Ok(None),
@@ -555,7 +563,7 @@ impl Socket for SocketImpl {
         .map_err(|err| IoError::Other(err, IoOperation::TakeError))
     }
 
-    #[instrument(skip(self), ret)]
+    #[instrument(skip(self), ret, level = "trace")]
     #[allow(unsafe_code)]
     fn icmp_error_info(&mut self) -> IoResult<IpAddr> {
         let icmp_error_info = self
@@ -579,15 +587,6 @@ impl Socket for SocketImpl {
             )),
         }
     }
-
-    // Interestingly, `Socket2` sockets don't seem to call `closesocket` on drop??
-    #[instrument(skip(self))]
-    fn close(&mut self) -> IoResult<()> {
-        syscall!(closesocket(self.inner.as_raw_socket() as _), |res| res
-            == SOCKET_ERROR)
-        .map_err(|err| IoError::Other(err, IoOperation::Close))
-        .map(|_| ())
-    }
 }
 
 // Note that we handle `WSAENOBUFS`, which can occurs when calling `send_to()`
@@ -627,9 +626,10 @@ impl From<ErrorKind> for StdIoError {
 /// is using a connectionless protocol, the address may not be available until I/O
 /// occurs on the socket."  We use `SIO_ROUTING_INTERFACE_QUERY` instead.
 #[allow(clippy::cast_sign_loss, clippy::redundant_closure_call)]
-#[instrument]
+#[instrument(level = "trace")]
 fn routing_interface_query(target: IpAddr) -> Result<IpAddr> {
-    let src: *mut c_void = [0; 1024].as_mut_ptr().cast();
+    let mut src_buf = [0; 1024];
+    let src: *mut c_void = src_buf.as_mut_ptr().cast();
     let mut bytes = 0;
     let socket = match target {
         IpAddr::V4(_) => SocketImpl::new_udp_dgram_socket_ipv4(),
@@ -754,13 +754,13 @@ fn socketaddr_to_sockaddr(socketaddr: SocketAddr) -> (SOCKADDR_STORAGE, i32) {
     (unsafe { sockaddr.storage }, size_of::<SockAddr>() as i32)
 }
 
-#[instrument(skip(adapters), ret)]
+#[instrument(skip(adapters), ret, level = "trace")]
 fn lookup_interface_addr(adapters: &Adapters, name: &str) -> Result<IpAddr> {
     adapters
         .iter()
         .find_map(|addr| {
             if addr.name.eq_ignore_ascii_case(name) {
-                Some(addr.addr)
+                addr.addr
             } else {
                 None
             }
@@ -850,8 +850,8 @@ mod adapter {
     pub struct AdapterAddress {
         /// The adapter friendly name.
         pub name: String,
-        /// The adapter `IpAddress`.
-        pub addr: IpAddr,
+        /// The first adapter uni-cast `IpAddr`, if any.
+        pub addr: Option<IpAddr>,
     }
 
     /// An iterator for `Adapters` which yields `AdapterAddress`
@@ -887,9 +887,13 @@ mod adapter {
                         .ok()?;
                     let addr = {
                         let first_unicast = (*self.next).FirstUnicastAddress;
-                        let socket_address = (*first_unicast).Address;
-                        let sockaddr = socket_address.lpSockaddr;
-                        sockaddrptr_to_ipaddr(sockaddr.cast()).ok()?
+                        if first_unicast.is_null() {
+                            None
+                        } else {
+                            let socket_address = (*first_unicast).Address;
+                            let sockaddr = socket_address.lpSockaddr;
+                            Some(sockaddrptr_to_ipaddr(sockaddr.cast()).ok()?)
+                        }
                     };
                     self.next = (*self.next).Next;
                     Some(AdapterAddress {
