@@ -1,4 +1,5 @@
 use crate::types::{Checksum, Flags, Port, RoundId, Sequence, TimeToLive, TraceId};
+use crate::TypeOfService;
 use std::net::IpAddr;
 use std::time::SystemTime;
 
@@ -89,12 +90,14 @@ impl Probe {
     }
 
     /// A response has been received and the probe is now complete.
+    #[allow(clippy::too_many_arguments)]
     #[must_use]
     pub(crate) const fn complete(
         self,
         host: IpAddr,
         received: SystemTime,
         icmp_packet_type: IcmpPacketType,
+        tos: Option<TypeOfService>,
         expected_udp_checksum: Option<Checksum>,
         actual_udp_checksum: Option<Checksum>,
         extensions: Option<Extensions>,
@@ -110,6 +113,7 @@ impl Probe {
             host,
             received,
             icmp_packet_type,
+            tos,
             expected_udp_checksum,
             actual_udp_checksum,
             extensions,
@@ -162,6 +166,8 @@ pub struct ProbeComplete {
     pub received: SystemTime,
     /// The type of ICMP response packet received for the probe.
     pub icmp_packet_type: IcmpPacketType,
+    /// The type of service (DSCP/ECN) of the original datagram.
+    pub tos: Option<TypeOfService>,
     /// The expected UDP checksum of the original datagram.
     pub expected_udp_checksum: Option<Checksum>,
     /// The actual UDP checksum of the original datagram.
@@ -281,48 +287,61 @@ pub struct ResponseData {
     pub recv: SystemTime,
     /// The `IpAddr` that responded to the probe.
     pub addr: IpAddr,
-    /// Information about the sequence number of the probe response.
-    pub resp_seq: ResponseSeq,
+    /// Protocol specific response information.
+    pub proto_resp: ProtocolResponse,
 }
 
 impl ResponseData {
-    pub const fn new(recv: SystemTime, addr: IpAddr, resp_seq: ResponseSeq) -> Self {
+    pub const fn new(recv: SystemTime, addr: IpAddr, proto_resp: ProtocolResponse) -> Self {
         Self {
             recv,
             addr,
-            resp_seq,
+            proto_resp,
         }
     }
 }
 
+/// Protocol specific response information.
+///
+/// This includes protocol specific information that is used to:
+///
+/// - determine the sequence number for matching the incoming probe response
+///   against the outgoing probe.
+/// - validate the probe response against the expected values and discard
+///   invalid responses.
+/// - record information from the probe Original Datagram such as the type of
+///   service (DSCP/ECN) and the expected UDP checksum.
 #[derive(Debug, Clone)]
-pub enum ResponseSeq {
-    Icmp(ResponseSeqIcmp),
-    Udp(ResponseSeqUdp),
-    Tcp(ResponseSeqTcp),
+pub enum ProtocolResponse {
+    Icmp(IcmpProtocolResponse),
+    Udp(UdpProtocolResponse),
+    Tcp(TcpProtocolResponse),
 }
 
 /// The data in the response to an ICMP probe.
 #[derive(Debug, Clone)]
-pub struct ResponseSeqIcmp {
+pub struct IcmpProtocolResponse {
     /// The ICMP identifier.
     pub identifier: u16,
     /// The ICMP sequence number.
     pub sequence: u16,
+    /// The type of service (DSCP/ECN) of the original datagram.
+    pub tos: Option<TypeOfService>,
 }
 
-impl ResponseSeqIcmp {
-    pub const fn new(identifier: u16, sequence: u16) -> Self {
+impl IcmpProtocolResponse {
+    pub const fn new(identifier: u16, sequence: u16, tos: Option<TypeOfService>) -> Self {
         Self {
             identifier,
             sequence,
+            tos,
         }
     }
 }
 
 /// The data in the response to a UDP probe.
 #[derive(Debug, Clone)]
-pub struct ResponseSeqUdp {
+pub struct UdpProtocolResponse {
     /// The IPv4 identifier.
     ///
     /// This will be the sequence number for IPv4/Dublin.
@@ -339,6 +358,8 @@ pub struct ResponseSeqUdp {
     ///
     /// This is used to validate the probe response matches the expected values.
     pub dest_port: u16,
+    /// The type of service (DSCP/ECN) of the original datagram.
+    pub tos: Option<TypeOfService>,
     /// The expected UDP checksum.
     ///
     /// This is calculated based on the data from the probe response and should
@@ -361,13 +382,14 @@ pub struct ResponseSeqUdp {
     pub has_magic: bool,
 }
 
-impl ResponseSeqUdp {
+impl UdpProtocolResponse {
     #[allow(clippy::too_many_arguments)]
     pub const fn new(
         identifier: u16,
         dest_addr: IpAddr,
         src_port: u16,
         dest_port: u16,
+        tos: Option<TypeOfService>,
         expected_udp_checksum: u16,
         actual_udp_checksum: u16,
         payload_len: u16,
@@ -378,6 +400,7 @@ impl ResponseSeqUdp {
             dest_addr,
             src_port,
             dest_port,
+            tos,
             expected_udp_checksum,
             actual_udp_checksum,
             payload_len,
@@ -388,7 +411,7 @@ impl ResponseSeqUdp {
 
 /// The data in the response to an TCP probe.
 #[derive(Debug, Clone)]
-pub struct ResponseSeqTcp {
+pub struct TcpProtocolResponse {
     /// The destination IP address.
     ///
     /// This is used to validate the probe response matches the expected values.
@@ -401,14 +424,22 @@ pub struct ResponseSeqTcp {
     ///
     /// This is used to validate the probe response matches the expected values.
     pub dest_port: u16,
+    /// The type of service (DSCP/ECN) of the original datagram.
+    pub tos: Option<TypeOfService>,
 }
 
-impl ResponseSeqTcp {
-    pub const fn new(dest_addr: IpAddr, src_port: u16, dest_port: u16) -> Self {
+impl TcpProtocolResponse {
+    pub const fn new(
+        dest_addr: IpAddr,
+        src_port: u16,
+        dest_port: u16,
+        tos: Option<TypeOfService>,
+    ) -> Self {
         Self {
             dest_addr,
             src_port,
             dest_port,
+            tos,
         }
     }
 }
