@@ -3,11 +3,13 @@ use crate::config::StrategyConfig;
 use crate::error::{Error, Result};
 use crate::net::Network;
 use crate::probe::{
-    ProbeStatus, Response, ResponseData, ResponseSeq, ResponseSeqIcmp, ResponseSeqTcp,
-    ResponseSeqUdp,
+    IcmpProtocolResponse, ProbeStatus, ProtocolResponse, Response, ResponseData,
+    TcpProtocolResponse, UdpProtocolResponse,
 };
 use crate::types::{Checksum, Sequence, TimeToLive, TraceId};
-use crate::{Extensions, IcmpPacketType, MultipathStrategy, PortDirection, Probe, Protocol};
+use crate::{
+    Extensions, IcmpPacketType, MultipathStrategy, PortDirection, Probe, Protocol, TypeOfService,
+};
 use std::net::IpAddr;
 use std::time::{Duration, SystemTime};
 use tracing::instrument;
@@ -55,7 +57,7 @@ pub struct Strategy<F> {
 }
 
 impl<F: Fn(&Round<'_>)> Strategy<F> {
-    #[instrument(skip_all)]
+    #[instrument(skip_all, level = "trace")]
     pub fn new(config: &StrategyConfig, publish: F) -> Self {
         tracing::debug!(?config);
         Self {
@@ -65,7 +67,7 @@ impl<F: Fn(&Round<'_>)> Strategy<F> {
     }
 
     /// Run a continuous trace and publish results.
-    #[instrument(skip(self, network))]
+    #[instrument(skip(self, network), level = "trace")]
     pub fn run<N: Network>(self, mut network: N) -> Result<()> {
         let mut state = TracerState::new(self.config);
         while !state.finished(self.config.max_rounds) {
@@ -87,7 +89,6 @@ impl<F: Fn(&Round<'_>)> Strategy<F> {
     ///         round
     ///     otherwise:
     ///       - the number of unknown-in-flight probes is lower than the maximum allowed
-    #[instrument(skip(self, network, st))]
     fn send_request<N: Network>(&self, network: &mut N, st: &mut TracerState) -> Result<()> {
         let can_send_ttl = if let Some(target_ttl) = st.target_ttl() {
             st.ttl() <= target_ttl
@@ -125,7 +126,7 @@ impl<F: Fn(&Round<'_>)> Strategy<F> {
                         }
                     }
                 }
-            };
+            }
         }
         Ok(())
     }
@@ -134,6 +135,7 @@ impl<F: Fn(&Round<'_>)> Strategy<F> {
     ///
     /// Some errors are transient and should not be considered fatal.  In these cases we mark the
     /// probe as failed and continue.
+    #[instrument(skip(network, st), level = "trace")]
     fn do_send<N: Network>(network: &mut N, st: &mut TracerState, probe: Probe) -> Result<()> {
         match network.send_probe(probe) {
             Ok(()) => Ok(()),
@@ -164,7 +166,6 @@ impl<F: Fn(&Round<'_>)> Strategy<F> {
     /// corresponding original `EchoRequest`.  Note that this may not be the greatest
     /// time-to-live that was sent in the round as the algorithm will send `EchoRequest` with
     /// larger time-to-live values before the `EchoReply` is received.
-    #[instrument(skip(self, network, st))]
     fn recv_response<N: Network>(&self, network: &mut N, st: &mut TracerState) -> Result<()> {
         let next = network.recv_probe()?;
         if let Some(resp) = next {
@@ -187,7 +188,6 @@ impl<F: Fn(&Round<'_>)> Strategy<F> {
     /// 3 - either:
     ///     A - the target has been found OR
     ///     B - the target has not been found and the round has exceeded the maximum round duration
-    #[instrument(skip(self, st))]
     fn update_round(&self, st: &mut TracerState) {
         let now = SystemTime::now();
         let round_duration = now.duration_since(st.round_start()).unwrap_or_default();
@@ -205,7 +205,7 @@ impl<F: Fn(&Round<'_>)> Strategy<F> {
     ///
     /// If the round completed without receiving an `EchoReply` from the target host then we also
     /// publish the next `ProbeStatus` which is assumed to represent the TTL of the target host.
-    #[instrument(skip(self, state))]
+    #[instrument(skip(self, state), level = "trace")]
     fn publish_trace(&self, state: &TracerState) {
         let max_received_ttl = if let Some(target_ttl) = state.target_ttl() {
             target_ttl
@@ -230,7 +230,7 @@ impl<F: Fn(&Round<'_>)> Strategy<F> {
     /// Check if the `TraceId` matches the expected value for this tracer.
     ///
     /// A special value of `0` is accepted for `udp` and `tcp` which do not have an identifier.
-    #[instrument(skip(self))]
+    #[instrument(skip(self), level = "trace")]
     fn check_trace_id(&self, trace_id: TraceId) -> bool {
         self.config.trace_identifier == trace_id || trace_id == TraceId(0)
     }
@@ -247,6 +247,7 @@ impl<F: Fn(&Round<'_>)> Strategy<F> {
     /// dest address match the expected values.
     ///
     /// For ICMP probe responses no additional checks are required.
+    #[instrument(skip(self), level = "trace")]
     fn validate(&self, resp: &ResponseData) -> bool {
         const fn validate_ports(
             port_direction: PortDirection,
@@ -262,9 +263,9 @@ impl<F: Fn(&Round<'_>)> Strategy<F> {
                 _ => false,
             }
         }
-        match resp.resp_seq {
-            ResponseSeq::Icmp(_) => true,
-            ResponseSeq::Udp(ResponseSeqUdp {
+        match resp.proto_resp {
+            ProtocolResponse::Icmp(_) => true,
+            ProtocolResponse::Udp(UdpProtocolResponse {
                 dest_addr,
                 src_port,
                 dest_port,
@@ -279,10 +280,11 @@ impl<F: Fn(&Round<'_>)> Strategy<F> {
                 };
                 check_dest_addr && check_ports && check_magic
             }
-            ResponseSeq::Tcp(ResponseSeqTcp {
+            ProtocolResponse::Tcp(TcpProtocolResponse {
                 dest_addr,
                 src_port,
                 dest_port,
+                ..
             }) => {
                 let check_ports = validate_ports(self.config.port_direction, src_port, dest_port);
                 let check_dest_addr = self.config.target_addr == dest_addr;
@@ -298,6 +300,7 @@ struct StrategyResponse {
     icmp_packet_type: IcmpPacketType,
     trace_id: TraceId,
     sequence: Sequence,
+    tos: Option<TypeOfService>,
     expected_udp_checksum: Option<Checksum>,
     actual_udp_checksum: Option<Checksum>,
     received: SystemTime,
@@ -310,14 +313,15 @@ impl From<(Response, &StrategyConfig)> for StrategyResponse {
     fn from((resp, config): (Response, &StrategyConfig)) -> Self {
         match resp {
             Response::TimeExceeded(data, code, exts) => {
-                let resp_seq = StrategyResponseSeq::from((data.resp_seq, config));
+                let proto_resp = ProtocolStrategyResponse::from((data.proto_resp, config));
                 let is_target = data.addr == config.target_addr;
                 Self {
                     icmp_packet_type: IcmpPacketType::TimeExceeded(code),
-                    trace_id: resp_seq.trace_id,
-                    sequence: resp_seq.sequence,
-                    expected_udp_checksum: resp_seq.expected_udp_checksum,
-                    actual_udp_checksum: resp_seq.actual_udp_checksum,
+                    trace_id: proto_resp.trace_id,
+                    sequence: proto_resp.sequence,
+                    tos: proto_resp.tos,
+                    expected_udp_checksum: proto_resp.expected_udp_checksum,
+                    actual_udp_checksum: proto_resp.actual_udp_checksum,
                     received: data.recv,
                     addr: data.addr,
                     is_target,
@@ -325,14 +329,15 @@ impl From<(Response, &StrategyConfig)> for StrategyResponse {
                 }
             }
             Response::DestinationUnreachable(data, code, exts) => {
-                let resp_seq = StrategyResponseSeq::from((data.resp_seq, config));
+                let proto_resp = ProtocolStrategyResponse::from((data.proto_resp, config));
                 let is_target = data.addr == config.target_addr;
                 Self {
                     icmp_packet_type: IcmpPacketType::Unreachable(code),
-                    trace_id: resp_seq.trace_id,
-                    sequence: resp_seq.sequence,
-                    expected_udp_checksum: resp_seq.expected_udp_checksum,
-                    actual_udp_checksum: resp_seq.actual_udp_checksum,
+                    trace_id: proto_resp.trace_id,
+                    sequence: proto_resp.sequence,
+                    tos: proto_resp.tos,
+                    expected_udp_checksum: proto_resp.expected_udp_checksum,
+                    actual_udp_checksum: proto_resp.actual_udp_checksum,
                     received: data.recv,
                     addr: data.addr,
                     is_target,
@@ -340,13 +345,14 @@ impl From<(Response, &StrategyConfig)> for StrategyResponse {
                 }
             }
             Response::EchoReply(data, code) => {
-                let resp_seq = StrategyResponseSeq::from((data.resp_seq, config));
+                let proto_resp = ProtocolStrategyResponse::from((data.proto_resp, config));
                 Self {
                     icmp_packet_type: IcmpPacketType::EchoReply(code),
-                    trace_id: resp_seq.trace_id,
-                    sequence: resp_seq.sequence,
-                    expected_udp_checksum: resp_seq.expected_udp_checksum,
-                    actual_udp_checksum: resp_seq.actual_udp_checksum,
+                    trace_id: proto_resp.trace_id,
+                    sequence: proto_resp.sequence,
+                    tos: proto_resp.tos,
+                    expected_udp_checksum: proto_resp.expected_udp_checksum,
+                    actual_udp_checksum: proto_resp.actual_udp_checksum,
                     received: data.recv,
                     addr: data.addr,
                     is_target: true,
@@ -354,13 +360,14 @@ impl From<(Response, &StrategyConfig)> for StrategyResponse {
                 }
             }
             Response::TcpReply(data) | Response::TcpRefused(data) => {
-                let resp_seq = StrategyResponseSeq::from((data.resp_seq, config));
+                let proto_resp = ProtocolStrategyResponse::from((data.proto_resp, config));
                 Self {
                     icmp_packet_type: IcmpPacketType::NotApplicable,
-                    trace_id: resp_seq.trace_id,
-                    sequence: resp_seq.sequence,
-                    expected_udp_checksum: resp_seq.expected_udp_checksum,
-                    actual_udp_checksum: resp_seq.actual_udp_checksum,
+                    trace_id: proto_resp.trace_id,
+                    sequence: proto_resp.sequence,
+                    tos: proto_resp.tos,
+                    expected_udp_checksum: proto_resp.expected_udp_checksum,
+                    actual_udp_checksum: proto_resp.actual_udp_checksum,
                     received: data.recv,
                     addr: data.addr,
                     is_target: true,
@@ -373,29 +380,33 @@ impl From<(Response, &StrategyConfig)> for StrategyResponse {
 
 /// Derived response sequence based on strategy config.
 #[derive(Debug)]
-struct StrategyResponseSeq {
+struct ProtocolStrategyResponse {
     trace_id: TraceId,
     sequence: Sequence,
+    tos: Option<TypeOfService>,
     expected_udp_checksum: Option<Checksum>,
     actual_udp_checksum: Option<Checksum>,
 }
 
-impl From<(ResponseSeq, &StrategyConfig)> for StrategyResponseSeq {
-    fn from((resp_seq, config): (ResponseSeq, &StrategyConfig)) -> Self {
-        match resp_seq {
-            ResponseSeq::Icmp(ResponseSeqIcmp {
+impl From<(ProtocolResponse, &StrategyConfig)> for ProtocolStrategyResponse {
+    fn from((proto_resp, config): (ProtocolResponse, &StrategyConfig)) -> Self {
+        match proto_resp {
+            ProtocolResponse::Icmp(IcmpProtocolResponse {
                 identifier,
                 sequence,
+                tos,
             }) => Self {
                 trace_id: TraceId(identifier),
                 sequence: Sequence(sequence),
+                tos,
                 expected_udp_checksum: None,
                 actual_udp_checksum: None,
             },
-            ResponseSeq::Udp(ResponseSeqUdp {
+            ProtocolResponse::Udp(UdpProtocolResponse {
                 identifier,
                 src_port,
                 dest_port,
+                tos,
                 expected_udp_checksum,
                 actual_udp_checksum,
                 payload_len,
@@ -427,13 +438,15 @@ impl From<(ResponseSeq, &StrategyConfig)> for StrategyResponseSeq {
                 Self {
                     trace_id: TraceId(0),
                     sequence: Sequence(sequence),
+                    tos,
                     expected_udp_checksum,
                     actual_udp_checksum,
                 }
             }
-            ResponseSeq::Tcp(ResponseSeqTcp {
+            ProtocolResponse::Tcp(TcpProtocolResponse {
                 src_port,
                 dest_port,
+                tos,
                 ..
             }) => {
                 let sequence = match config.port_direction {
@@ -443,6 +456,7 @@ impl From<(ResponseSeq, &StrategyConfig)> for StrategyResponseSeq {
                 Self {
                     trace_id: TraceId(0),
                     sequence: Sequence(sequence),
+                    tos,
                     expected_udp_checksum: None,
                     actual_udp_checksum: None,
                 }
@@ -594,11 +608,12 @@ mod tests {
     #[test]
     fn test_icmp_response() {
         let config = StrategyConfig::default();
-        let resp_seq = ResponseSeq::Icmp(ResponseSeqIcmp {
+        let proto_resp = ProtocolResponse::Icmp(IcmpProtocolResponse {
             identifier: 1234,
             sequence: 33434,
+            tos: Some(TypeOfService(0)),
         });
-        let strategy_resp = StrategyResponseSeq::from((resp_seq, &config));
+        let strategy_resp = ProtocolStrategyResponse::from((proto_resp, &config));
         assert_eq!(strategy_resp.trace_id, TraceId(1234));
         assert_eq!(strategy_resp.sequence, Sequence(33434));
     }
@@ -610,17 +625,18 @@ mod tests {
             port_direction: PortDirection::FixedSrc(Port(5000)),
             ..Default::default()
         };
-        let resp_seq = ResponseSeq::Udp(ResponseSeqUdp {
+        let proto_resp = ProtocolResponse::Udp(UdpProtocolResponse {
             identifier: 0,
             dest_addr: IpAddr::V4(Ipv4Addr::new(10, 0, 0, 1)),
             src_port: 5000,
             dest_port: 33434,
+            tos: Some(TypeOfService(0)),
             expected_udp_checksum: 0,
             actual_udp_checksum: 0,
             payload_len: 0,
             has_magic: false,
         });
-        let strategy_resp = StrategyResponseSeq::from((resp_seq, &config));
+        let strategy_resp = ProtocolStrategyResponse::from((proto_resp, &config));
         assert_eq!(strategy_resp.trace_id, TraceId(0));
         assert_eq!(strategy_resp.sequence, Sequence(33434));
     }
@@ -632,17 +648,18 @@ mod tests {
             port_direction: PortDirection::FixedDest(Port(5000)),
             ..Default::default()
         };
-        let resp_seq = ResponseSeq::Udp(ResponseSeqUdp {
+        let proto_resp = ProtocolResponse::Udp(UdpProtocolResponse {
             identifier: 0,
             dest_addr: IpAddr::V4(Ipv4Addr::new(10, 0, 0, 1)),
             src_port: 33434,
             dest_port: 5000,
+            tos: Some(TypeOfService(0)),
             expected_udp_checksum: 0,
             actual_udp_checksum: 0,
             payload_len: 0,
             has_magic: false,
         });
-        let strategy_resp = StrategyResponseSeq::from((resp_seq, &config));
+        let strategy_resp = ProtocolStrategyResponse::from((proto_resp, &config));
         assert_eq!(strategy_resp.trace_id, TraceId(0));
         assert_eq!(strategy_resp.sequence, Sequence(33434));
     }
@@ -655,17 +672,18 @@ mod tests {
             port_direction: PortDirection::FixedSrc(Port(5000)),
             ..Default::default()
         };
-        let resp_seq = ResponseSeq::Udp(ResponseSeqUdp {
+        let proto_resp = ProtocolResponse::Udp(UdpProtocolResponse {
             identifier: 33434,
             dest_addr: IpAddr::V4(Ipv4Addr::new(10, 0, 0, 1)),
             src_port: 5000,
             dest_port: 35000,
+            tos: Some(TypeOfService(0)),
             expected_udp_checksum: 33434,
             actual_udp_checksum: 33434,
             payload_len: 0,
             has_magic: false,
         });
-        let strategy_resp = StrategyResponseSeq::from((resp_seq, &config));
+        let strategy_resp = ProtocolStrategyResponse::from((proto_resp, &config));
         assert_eq!(strategy_resp.trace_id, TraceId(0));
         assert_eq!(strategy_resp.sequence, Sequence(33434));
     }
@@ -678,17 +696,18 @@ mod tests {
             port_direction: PortDirection::FixedSrc(Port(5000)),
             ..Default::default()
         };
-        let resp_seq = ResponseSeq::Udp(ResponseSeqUdp {
+        let proto_resp = ProtocolResponse::Udp(UdpProtocolResponse {
             identifier: 33434,
             dest_addr: IpAddr::V4(Ipv4Addr::new(10, 0, 0, 1)),
             src_port: 5000,
             dest_port: 35000,
+            tos: Some(TypeOfService(0)),
             expected_udp_checksum: 0,
             actual_udp_checksum: 0,
             payload_len: 0,
             has_magic: false,
         });
-        let strategy_resp = StrategyResponseSeq::from((resp_seq, &config));
+        let strategy_resp = ProtocolStrategyResponse::from((proto_resp, &config));
         assert_eq!(strategy_resp.trace_id, TraceId(0));
         assert_eq!(strategy_resp.sequence, Sequence(33434));
     }
@@ -702,17 +721,18 @@ mod tests {
             port_direction: PortDirection::FixedSrc(Port(5000)),
             ..Default::default()
         };
-        let resp_seq = ResponseSeq::Udp(ResponseSeqUdp {
+        let proto_resp = ProtocolResponse::Udp(UdpProtocolResponse {
             identifier: 0,
             dest_addr: IpAddr::V6("::1".parse().unwrap()),
             src_port: 5000,
             dest_port: 35000,
+            tos: Some(TypeOfService(0)),
             expected_udp_checksum: 0,
             actual_udp_checksum: 0,
             payload_len: 55,
             has_magic: true,
         });
-        let strategy_resp = StrategyResponseSeq::from((resp_seq, &config));
+        let strategy_resp = ProtocolStrategyResponse::from((proto_resp, &config));
         assert_eq!(strategy_resp.trace_id, TraceId(0));
         assert_eq!(strategy_resp.sequence, Sequence(33489));
     }
@@ -724,17 +744,18 @@ mod tests {
             port_direction: PortDirection::FixedDest(Port(80)),
             ..Default::default()
         };
-        let resp_seq = ResponseSeq::Udp(ResponseSeqUdp {
+        let proto_resp = ProtocolResponse::Udp(UdpProtocolResponse {
             identifier: 0,
             dest_addr: IpAddr::V4(Ipv4Addr::new(10, 0, 0, 1)),
             src_port: 33434,
             dest_port: 80,
+            tos: Some(TypeOfService(0)),
             expected_udp_checksum: 0,
             actual_udp_checksum: 0,
             payload_len: 0,
             has_magic: false,
         });
-        let strategy_resp = StrategyResponseSeq::from((resp_seq, &config));
+        let strategy_resp = ProtocolStrategyResponse::from((proto_resp, &config));
         assert_eq!(strategy_resp.trace_id, TraceId(0));
         assert_eq!(strategy_resp.sequence, Sequence(33434));
     }
@@ -746,17 +767,18 @@ mod tests {
             port_direction: PortDirection::FixedSrc(Port(5000)),
             ..Default::default()
         };
-        let resp_seq = ResponseSeq::Udp(ResponseSeqUdp {
+        let proto_resp = ProtocolResponse::Udp(UdpProtocolResponse {
             identifier: 0,
             dest_addr: IpAddr::V4(Ipv4Addr::new(10, 0, 0, 1)),
             src_port: 5000,
             dest_port: 33434,
+            tos: Some(TypeOfService(0)),
             expected_udp_checksum: 0,
             actual_udp_checksum: 0,
             payload_len: 0,
             has_magic: false,
         });
-        let strategy_resp = StrategyResponseSeq::from((resp_seq, &config));
+        let strategy_resp = ProtocolStrategyResponse::from((proto_resp, &config));
         assert_eq!(strategy_resp.trace_id, TraceId(0));
         assert_eq!(strategy_resp.sequence, Sequence(33434));
     }
@@ -786,7 +808,12 @@ mod tests {
                     ResponseData::new(
                         SystemTime::now(),
                         target_addr,
-                        ResponseSeq::Tcp(ResponseSeqTcp::new(target_addr, sequence, 80)),
+                        ProtocolResponse::Tcp(TcpProtocolResponse::new(
+                            target_addr,
+                            sequence,
+                            80,
+                            None,
+                        )),
                     ),
                     IcmpPacketCode(1),
                     None,
@@ -800,7 +827,12 @@ mod tests {
                 Ok(Some(Response::TcpRefused(ResponseData::new(
                     SystemTime::now(),
                     target_addr,
-                    ResponseSeq::Tcp(ResponseSeqTcp::new(target_addr, sequence, 80)),
+                    ProtocolResponse::Tcp(TcpProtocolResponse::new(
+                        target_addr,
+                        sequence,
+                        80,
+                        None,
+                    )),
                 ))))
             });
 
@@ -824,9 +856,10 @@ mod tests {
         ResponseData::new(
             now,
             IpAddr::V4(Ipv4Addr::new(1, 2, 3, 4)),
-            ResponseSeq::Icmp(ResponseSeqIcmp {
+            ProtocolResponse::Icmp(IcmpProtocolResponse {
                 identifier: 0,
                 sequence: 33434,
+                tos: Some(TypeOfService(0)),
             }),
         )
     }
@@ -977,7 +1010,7 @@ mod state {
         ///
         /// We post-increment `ttl` here and so in practice we only allow `ttl` values in the range
         /// `1..254` to allow us to use a `u8`.
-        #[instrument(skip(self))]
+        #[instrument(skip(self), level = "trace")]
         pub fn next_probe(&mut self, sent: SystemTime) -> Probe {
             let (src_port, dest_port, identifier, flags) = self.probe_data();
             let probe = Probe::new(
@@ -1008,7 +1041,7 @@ mod state {
         /// then afterward:
         /// - The `ProbeStatus` at sequence `3` will be set to `Skipped` state
         /// - A new `ProbeStatus` will be created at sequence `4` with a `ttl` of `5`
-        #[instrument(skip(self))]
+        #[instrument(skip(self), level = "trace")]
         pub fn reissue_probe(&mut self, sent: SystemTime) -> Probe {
             let probe_index = usize::from(self.sequence - self.round_sequence);
             self.buffer[probe_index - 1] = ProbeStatus::Skipped;
@@ -1030,7 +1063,7 @@ mod state {
         }
 
         /// Mark the `ProbeStatus` at the current `sequence` as failed.
-        #[instrument(skip(self))]
+        #[instrument(skip(self), level = "trace")]
         pub fn fail_probe(&mut self) {
             let probe_index = usize::from(self.sequence - self.round_sequence);
             let probe = self.buffer[probe_index - 1].clone();
@@ -1160,7 +1193,7 @@ mod state {
         /// overwriting the state with stale values.  We may also receive multiple replies
         /// from the target host with differing time-to-live values and so must ensure we
         /// use the time-to-live with the lowest sequence number.
-        #[instrument(skip(self))]
+        #[instrument(skip(self), level = "trace")]
         pub fn complete_probe(&mut self, resp: StrategyResponse) {
             // Retrieve and update the `ProbeStatus` at `sequence`.
             let probe = self.probe_at(resp.sequence);
@@ -1183,6 +1216,7 @@ mod state {
                 resp.addr,
                 resp.received,
                 resp.icmp_packet_type,
+                resp.tos,
                 resp.expected_udp_checksum,
                 resp.actual_udp_checksum,
                 resp.exts,
@@ -1226,7 +1260,7 @@ mod state {
         /// If, during the round which just completed, we went above the max sequence number then we
         /// reset it here. We do this here to avoid having to deal with the sequence number
         /// wrapping during a round, which is more problematic.
-        #[instrument(skip(self))]
+        #[instrument(skip(self), level = "trace")]
         pub fn advance_round(&mut self, first_ttl: TimeToLive) {
             if self.sequence >= self.max_sequence() {
                 self.sequence = self.config.initial_sequence;
@@ -1265,6 +1299,7 @@ mod state {
         use super::*;
         use crate::probe::{IcmpPacketCode, IcmpPacketType};
         use crate::types::MaxInflight;
+        use crate::TypeOfService;
         use rand::Rng;
         use std::net::{IpAddr, Ipv4Addr};
         use std::time::Duration;
@@ -1307,6 +1342,7 @@ mod state {
                 icmp_packet_type: IcmpPacketType::TimeExceeded(IcmpPacketCode(1)),
                 trace_id: TraceId(0),
                 sequence: Sequence(33434),
+                tos: Some(TypeOfService(0)),
                 expected_udp_checksum: None,
                 actual_udp_checksum: None,
                 received: received_1,
@@ -1382,6 +1418,7 @@ mod state {
                 icmp_packet_type: IcmpPacketType::TimeExceeded(IcmpPacketCode(1)),
                 trace_id: TraceId(0),
                 sequence: Sequence(33435),
+                tos: Some(TypeOfService(0)),
                 expected_udp_checksum: None,
                 actual_udp_checksum: None,
                 received: received_2,
@@ -1417,6 +1454,7 @@ mod state {
                 icmp_packet_type: IcmpPacketType::EchoReply(IcmpPacketCode(0)),
                 trace_id: TraceId(0),
                 sequence: Sequence(33436),
+                tos: Some(TypeOfService(0)),
                 expected_udp_checksum: None,
                 actual_udp_checksum: None,
                 received: received_3,
@@ -1529,9 +1567,9 @@ mod state {
             let total_rounds = 2000;
             let max_probe_per_round = 20;
             let mut state = TracerState::new(cfg(Sequence(33434)));
-            let mut rng = rand::thread_rng();
+            let mut rng = rand::rng();
             for _ in 0..total_rounds {
-                for _ in 0..rng.gen_range(0..max_probe_per_round) {
+                for _ in 0..rng.random_range(0..max_probe_per_round) {
                     state.next_probe(SystemTime::now());
                 }
                 state.advance_round(TimeToLive(1));
