@@ -20,14 +20,17 @@ pub enum ResolveMethod {
 /// How to resolve IP addresses.
 #[derive(Debug, Copy, Clone, Eq, PartialEq)]
 pub enum IpAddrFamily {
-    /// Lookup Ipv4 only.
+    /// Lookup IPv4 only.
     Ipv4Only,
-    /// Lookup Ipv6 only.
+    /// Lookup IPv6 only.
     Ipv6Only,
-    /// Lookup Ipv6 with a fallback to Ipv4
+    /// Lookup IPv6 with a fallback to IPv4.
     Ipv6thenIpv4,
-    /// Lookup Ipv4 with a fallback to Ipv6
+    /// Lookup IPv4 with a fallback to IPv6.
     Ipv4thenIpv6,
+    /// Use the first IP address returned by the OS resolver when using `ResolveMethod::System`,
+    /// otherwise lookup IPv6 with a fallback to IPv4.
+    System,
 }
 
 impl Display for IpAddrFamily {
@@ -37,6 +40,7 @@ impl Display for IpAddrFamily {
             Self::Ipv6Only => write!(f, "Ipv6Only"),
             Self::Ipv6thenIpv4 => write!(f, "Ipv6thenIpv4"),
             Self::Ipv4thenIpv6 => write!(f, "Ipv4thenIpv6"),
+            Self::System => write!(f, "System"),
         }
     }
 }
@@ -71,19 +75,15 @@ impl Resolver for DnsResolver {
     fn lookup(&self, hostname: impl AsRef<str>) -> Result<ResolvedIpAddrs> {
         self.inner.lookup(hostname.as_ref())
     }
-    #[must_use]
     fn reverse_lookup(&self, addr: impl Into<IpAddr>) -> DnsEntry {
         self.inner.reverse_lookup(addr.into(), false, false)
     }
-    #[must_use]
     fn reverse_lookup_with_asinfo(&self, addr: impl Into<IpAddr>) -> DnsEntry {
         self.inner.reverse_lookup(addr.into(), true, false)
     }
-    #[must_use]
     fn lazy_reverse_lookup(&self, addr: impl Into<IpAddr>) -> DnsEntry {
         self.inner.reverse_lookup(addr.into(), false, true)
     }
-    #[must_use]
     fn lazy_reverse_lookup_with_asinfo(&self, addr: impl Into<IpAddr>) -> DnsEntry {
         self.inner.reverse_lookup(addr.into(), true, true)
     }
@@ -98,6 +98,7 @@ mod inner {
     use hickory_resolver::error::{ResolveError, ResolveErrorKind};
     use hickory_resolver::proto::error::ProtoError;
     use hickory_resolver::proto::rr::RecordType;
+    use hickory_resolver::system_conf::read_system_conf;
     use hickory_resolver::{Name, Resolver};
     use itertools::{Either, Itertools};
     use parking_lot::RwLock;
@@ -166,15 +167,24 @@ mod inner {
                 DnsProvider::DnsLookup
             } else {
                 let mut options = ResolverOpts::default();
-                options.timeout = config.timeout;
-                options.ip_strategy = match config.addr_family {
+                #[allow(clippy::match_same_arms)]
+                let ip_strategy = match config.addr_family {
                     IpAddrFamily::Ipv4Only => LookupIpStrategy::Ipv4Only,
                     IpAddrFamily::Ipv6Only => LookupIpStrategy::Ipv6Only,
                     IpAddrFamily::Ipv6thenIpv4 => LookupIpStrategy::Ipv6thenIpv4,
                     IpAddrFamily::Ipv4thenIpv6 => LookupIpStrategy::Ipv4thenIpv6,
+                    // see issue #1469
+                    IpAddrFamily::System => LookupIpStrategy::Ipv6thenIpv4,
                 };
+                options.timeout = config.timeout;
+                options.ip_strategy = ip_strategy;
                 let res = match config.resolve_method {
-                    ResolveMethod::Resolv => Resolver::from_system_conf(),
+                    ResolveMethod::Resolv => {
+                        let (resolver_cfg, mut options) = read_system_conf()?;
+                        options.timeout = config.timeout;
+                        options.ip_strategy = ip_strategy;
+                        Resolver::new(resolver_cfg, options)
+                    }
                     ResolveMethod::Google => Resolver::new(ResolverConfig::google(), options),
                     ResolveMethod::Cloudflare => {
                         Resolver::new(ResolverConfig::cloudflare(), options)
@@ -204,6 +214,12 @@ mod inner {
         }
 
         pub(super) fn lookup(&self, hostname: &str) -> Result<ResolvedIpAddrs> {
+            fn partition(all: Vec<IpAddr>) -> (Vec<IpAddr>, Vec<IpAddr>) {
+                all.into_iter().partition_map(|ip| match ip {
+                    IpAddr::V4(_) => Either::Left(ip),
+                    IpAddr::V6(_) => Either::Right(ip),
+                })
+            }
             match &self.provider {
                 DnsProvider::TrustDns(resolver) => Ok(resolver
                     .lookup_ip(hostname)
@@ -211,15 +227,11 @@ mod inner {
                     .iter()
                     .collect::<Vec<_>>()),
                 DnsProvider::DnsLookup => {
-                    let (ipv4, ipv6): (Vec<_>, Vec<_>) = dns_lookup::lookup_host(hostname)
-                        .map_err(|err| Error::LookupFailed(Box::new(err)))?
-                        .into_iter()
-                        .partition_map(|ip| match ip {
-                            IpAddr::V4(_) => Either::Left(ip),
-                            IpAddr::V6(_) => Either::Right(ip),
-                        });
+                    let all = dns_lookup::lookup_host(hostname)
+                        .map_err(|err| Error::LookupFailed(Box::new(err)))?;
                     Ok(match self.config.addr_family {
                         IpAddrFamily::Ipv4Only => {
+                            let (ipv4, _) = partition(all);
                             if ipv4.is_empty() {
                                 vec![]
                             } else {
@@ -227,6 +239,7 @@ mod inner {
                             }
                         }
                         IpAddrFamily::Ipv6Only => {
+                            let (_, ipv6) = partition(all);
                             if ipv6.is_empty() {
                                 vec![]
                             } else {
@@ -234,6 +247,7 @@ mod inner {
                             }
                         }
                         IpAddrFamily::Ipv6thenIpv4 => {
+                            let (ipv4, ipv6) = partition(all);
                             if ipv6.is_empty() {
                                 ipv4
                             } else {
@@ -241,12 +255,14 @@ mod inner {
                             }
                         }
                         IpAddrFamily::Ipv4thenIpv6 => {
+                            let (ipv4, ipv6) = partition(all);
                             if ipv4.is_empty() {
                                 ipv6
                             } else {
                                 ipv4
                             }
                         }
+                        IpAddrFamily::System => all,
                     })
                 }
             }
@@ -300,7 +316,7 @@ mod inner {
                 _ => {}
             }
 
-            // If the entry exists but has timed out, then set it as DnsEntry::Pending and enqueue
+            // If the entry exists but has timed out, then set it as `DnsEntry::Pending` and enqueue
             // it again.
             if let DnsEntry::Timeout(addr) = dns_entry.entry {
                 *self
@@ -315,7 +331,7 @@ mod inner {
 
             // If this is a newly added `DnsEntry` then send it to the channel to be resolved in the
             // background.  We do this after the above to ensure we aren't holding the
-            // lock on the cache, which is usd by the resolver and so would deadlock.
+            // lock on the cache, which is used by the resolver and so would deadlock.
             if enqueue {
                 if self
                     .tx
