@@ -860,7 +860,7 @@ pub mod time_exceeded {
         }
 
         fn split_payload_extension(&self) -> (&[u8], Option<&[u8]>) {
-            let length = usize::from(self.get_length() * 8);
+            let length = usize::from(self.get_length()) * 8;
             let icmp_payload = &self.buf.as_slice()[Self::minimum_packet_size()..];
             split(length, icmp_payload)
         }
@@ -959,6 +959,19 @@ pub mod time_exceeded {
             assert!(packet.payload().is_empty());
         }
 
+        #[test]
+        fn test_view_large() {
+            let mut buf = [0x0_u8; 128];
+            buf[..8].copy_from_slice(&[0x03, 0x00, 0xf4, 0xee, 0x20, 0x00, 0x00, 0x00]);
+            let packet = TimeExceededPacket::new_view(&buf).unwrap();
+            assert_eq!(IcmpType::TimeExceeded, packet.get_icmp_type());
+            assert_eq!(IcmpCode(0), packet.get_icmp_code());
+            assert_eq!(62702, packet.get_checksum());
+            assert_eq!(32, packet.get_length());
+            assert_eq!(&[0x0_u8; 120], packet.payload());
+            assert_eq!(None, packet.extension());
+        }
+
         #[test]
         fn test_new_insufficient_buffer() {
             const SIZE: usize = TimeExceededPacket::minimum_packet_size();
@@ -1117,7 +1130,7 @@ pub mod destination_unreachable {
             // From rfc4884:
             //
             // "For ICMPv6 messages, the length attribute represents 64-bit words"
-            let length = usize::from(self.get_length() * 8);
+            let length = usize::from(self.get_length()) * 8;
             let icmp_payload = &self.buf.as_slice()[Self::minimum_packet_size()..];
             split(length, icmp_payload)
         }
@@ -1217,6 +1230,19 @@ pub mod destination_unreachable {
             assert!(packet.payload().is_empty());
         }
 
+        #[test]
+        fn test_view_large() {
+            let mut buf = [0x0_u8; 128];
+            buf[..8].copy_from_slice(&[0x01, 0x03, 0xdf, 0xdc, 0x20, 0x00, 0x00, 0x00]);
+            let packet = DestinationUnreachablePacket::new_view(&buf).unwrap();
+            assert_eq!(IcmpType::DestinationUnreachable, packet.get_icmp_type());
+            assert_eq!(IcmpCode(3), packet.get_icmp_code());
+            assert_eq!(57308, packet.get_checksum());
+            assert_eq!(32, packet.get_length());
+            assert_eq!(&[0x0_u8; 120], packet.payload());
+            assert_eq!(None, packet.extension());
+        }
+
         #[test]
         fn test_new_insufficient_buffer() {
             const SIZE: usize = DestinationUnreachablePacket::minimum_packet_size();
