@@ -79,6 +79,11 @@ impl<'a> Ipv4Packet<'a> {
         self.buf.read(ECN_OFFSET) & 0x3
     }
 
+    #[must_use]
+    pub fn get_tos(&self) -> u8 {
+        (self.get_dscp() << 2) | self.get_ecn()
+    }
+
     #[must_use]
     pub fn get_total_length(&self) -> u16 {
         u16::from_be_bytes(self.buf.get_bytes(TOTAL_LENGTH_OFFSET))
@@ -146,6 +151,11 @@ impl<'a> Ipv4Packet<'a> {
         *self.buf.write(ECN_OFFSET) = (self.buf.read(ECN_OFFSET) & 0xfc) | (val & 0x3);
     }
 
+    pub fn set_tos(&mut self, val: u8) {
+        self.set_dscp((val & 0xfc) >> 2);
+        self.set_ecn(val & 0x3);
+    }
+
     pub fn set_total_length(&mut self, val: u16) {
         self.buf.set_bytes(TOTAL_LENGTH_OFFSET, val.to_be_bytes());
     }
@@ -284,7 +294,7 @@ mod tests {
         let mut packet = Ipv4Packet::new(&mut buf).unwrap();
         packet.set_dscp(63);
         assert_eq!(63, packet.get_dscp());
-        assert_eq!([0x00, 0xFC], packet.packet()[..2]);
+        assert_eq!([0xFC], packet.packet()[1..2]);
     }
 
     #[test]
@@ -293,7 +303,7 @@ mod tests {
         let mut packet = Ipv4Packet::new(&mut buf).unwrap();
         packet.set_ecn(3);
         assert_eq!(3, packet.get_ecn());
-        assert_eq!([0x00, 0x03], packet.packet()[..2]);
+        assert_eq!([0x03], packet.packet()[1..2]);
     }
 
     #[test]
@@ -304,7 +314,23 @@ mod tests {
         packet.set_ecn(3);
         assert_eq!(63, packet.get_dscp());
         assert_eq!(3, packet.get_ecn());
-        assert_eq!([0x00, 0xFF], packet.packet()[..2]);
+        assert_eq!([0xFF], packet.packet()[1..2]);
+    }
+
+    #[test]
+    fn test_tos() {
+        let mut buf = [0_u8; Ipv4Packet::minimum_packet_size()];
+        let mut packet = Ipv4Packet::new(&mut buf).unwrap();
+        packet.set_tos(224);
+        assert_eq!(224, packet.get_tos());
+        assert_eq!(56, packet.get_dscp());
+        assert_eq!(0, packet.get_ecn());
+        assert_eq!([0xE0], packet.packet()[1..2]);
+        packet.set_tos(255);
+        assert_eq!(255, packet.get_tos());
+        assert_eq!(63, packet.get_dscp());
+        assert_eq!(3, packet.get_ecn());
+        assert_eq!([0xFF], packet.packet()[1..2]);
     }
 
     #[test]
