@@ -65,7 +65,7 @@ pub struct Args {
     )]
     pub icmp: bool,
 
-    /// The address family [default: Ipv4thenIpv6]
+    /// The address family [default: ipv4-then-ipv6]
     #[arg(value_enum, short = 'F', long)]
     pub addr_family: Option<AddressFamilyConfig>,
 
@@ -144,7 +144,7 @@ pub struct Args {
     #[arg(long)]
     pub payload_pattern: Option<u8>,
 
-    /// The TOS (i.e. DSCP+ECN) IP header value (TCP and UDP only) [default: 0]
+    /// The TOS (i.e. DSCP+ECN) IP header value (IPv4 only) [default: 0]
     #[arg(short = 'Q', long)]
     pub tos: Option<u8>,
 
@@ -226,6 +226,12 @@ pub struct Args {
     #[arg(long)]
     pub tui_locale: Option<String>,
 
+    /// The timezone to use for the TUI [default: auto]
+    ///
+    /// The timezone must be a valid IANA timezone identifier.
+    #[arg(long)]
+    pub tui_timezone: Option<String>,
+
     /// The TUI theme colors [item=color,item=color,..]
     #[arg(long, value_delimiter(','), value_parser = parse_tui_theme_color_value)]
     pub tui_theme_colors: Vec<(TuiThemeItem, TuiColor)>,
