@@ -28,6 +28,7 @@ const DEFAULT_HIDDEN_CONFIG_FILE: &str = ".trippy.toml";
 ///     - the current directory
 ///     - the user home directory
 ///     - the XDG config directory (Unix only): `$XDG_CONFIG_HOME` or `~/.config`
+///     - the XDG app config directory (Unix only): `$XDG_CONFIG_HOME/trippy` or `~/.config/trippy`
 ///     - the Windows data directory (Windows only): `%APPDATA%`
 ///
 /// Note that only the first config file found is used, no attempt is
@@ -42,6 +43,8 @@ pub fn read_default_config_file() -> anyhow::Result<Option<ConfigFile>> {
             Ok(Some(file))
         } else if let Some(file) = read_files(basedirs.config_dir())? {
             Ok(Some(file))
+        } else if let Some(file) = read_files(basedirs.config_dir().join("trippy"))? {
+            Ok(Some(file))
         } else {
             Ok(None)
         }
@@ -248,6 +251,7 @@ pub struct ConfigTui {
     pub geoip_mmdb_file: Option<String>,
     pub tui_custom_columns: Option<String>,
     pub tui_locale: Option<String>,
+    pub tui_timezone: Option<String>,
     #[serde(rename = "tui-max-samples")]
     pub deprecated_tui_max_samples: Option<usize>,
     #[serde(rename = "tui-max-flows")]
@@ -267,6 +271,7 @@ impl Default for ConfigTui {
             tui_geoip_mode: Some(super::constants::DEFAULT_TUI_GEOIP_MODE),
             tui_max_addrs: Some(super::constants::DEFAULT_TUI_MAX_ADDRS),
             tui_locale: None,
+            tui_timezone: None,
             geoip_mmdb_file: None,
             deprecated_tui_max_samples: None,
             deprecated_tui_max_flows: None,
